# -*- coding: utf-8 -*-

import logging
import werkzeug

from odoo.addons.saas_worker.util import from_role, email_on_exception
from odoo.addons.saas_worker.compat import route, http, request
from odoo.addons.saas_worker.const import SAAS_BASE_URL
from odoo.tools.parse_version import parse_version

_logger = logging.getLogger(__name__)


def email_notify(template_ref, db_url, email_to, mode, state):
    template = request.env.ref(template_ref)
    context = {
        'email_to': email_to,
        'db_name': db_url.split('://')[-1],
        'db_url': db_url,
        'mode': mode,
    }
    _logger.info('sent migration-notify email to %s for %s' % (email_to, db_url))
    template.sudo().with_context(context).send_mail(False, raise_exception=False)


class MigrationController(http.Controller):

    @from_role('trial')
    @email_on_exception(allowed_exceptions=(werkzeug.exceptions.HTTPException,))
    @route('/odoo-enterprise/migration-notify', auth='none', csrf=False, readonly=False)
    def notify(self, **kw):
        db_url = kw.get('db_url')
        mode = kw.get('mode')
        state = kw.get('state')
        email_to = kw.get('email_to')
        if state == 'done':
            template_ref = 'openerp_enterprise.migration_done'
        else:
            return ''

        if not db_url:
            mig_uuid = kw.get('copy', kw.get('db'))
            db = request.env['openerp.enterprise.database'].sudo().search([('db_uuid', '=', mig_uuid)])
            db_url = db.url

        email_notify(template_ref, db_url, email_to, mode, state)
        return ''

    @route('/odoo-enterprise/getlastupgradetest', auth='user', type='json', readonly=True)
    def getlastupgradetest(self, **kw):
        Database = request.env['openerp.enterprise.database'].sudo()
        main_db = Database.search([('db_uuid', '=', kw.get('dbuuid')), ('registered_user_ids.user_id', '=', request.uid)])
        if not main_db:
            raise werkzeug.exceptions.NotFound()
        child_dbs = Database.search([('parent_id', '=', main_db.id), ('hosting', '=', 'saas')], order="create_date DESC")
        main_db_version = parse_version(main_db.version)
        for db in child_dbs:
            if main_db_version < parse_version(db.version):
                return {
                    'url': db.url,
                    'version': db.version,
                    'date': db.create_date.isoformat()
                }
        return {}
